<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Class for notification users via email
 *
 */
class MailNotificator extends Notificator
{
    /**
     * @var string cache time for user model
     */
    public $cacheTime = 3600;

    /**
     * @var array emails to which notification sent by default
     */
    public $defaultEmails = array();

    /**
     * @var array PhpMailer instance properties
     */
    public $phpMailer = array();

    /**
     * Create instance of mail object
     *
     * @return PHPMailer
     */
    public function createMail()
    {
        $mail = new PHPMailer(true);
        foreach ($this->phpMailer as $prop => $value) {
            $mail->$prop = $value;
        }
        return $mail;
    }

    /**
     * Check if email recipient is allowed to get mails
     *
     * @param EmailRecipient $recipient recipient
     *
     * @return bool
     */
    protected function validate($recipient)
    {
        if ($recipient instanceof EmailRecipient) {
            if (!$recipient->skipValidation && $userRecipient = User::model()->cache(Yii::app()->notificationManager->cacheTime)->with('profile')->findByAttributes(array('email' => $recipient->to))) {
                if ($userRecipient->profile->send_me_notifications == false) {
                    Yii::trace("Email skipped {$recipient->to}", 'MailNotificator');
                    return false;
                }
            }
            Yii::trace("Email approved {$recipient->to}", 'MailNotificator');
            return true;
        }
        return false;
    }

    /**
     * Send notification to all recipients who is instance of EmailRecipient
     *
     * @param Notification $notification occurred notification
     *
     * @return void
     */
    public function notify(Notification $notification)
    {
        foreach ($notification->recipients as $recipient) {
            if ($this->validate($recipient)) {
                $mail = $this->createMail();
                if ($recipient->from) {
                    $mail->SetFrom($recipient->from);
                }
                $mail->AddAddress($recipient->to);
                $mail->Subject = $notification->subject;
                $mail->MsgHTML($notification->message);
                try {
                    $mail->Send();
                    Yii::trace("Mail sent success. Subject:{$notification->subject}. To:{$recipient->to}, From: {$recipient->from}", 'MailNotificator');
                } catch (Exception $e) {
                    Yii::log($e->getMessage(), CLogger::LEVEL_ERROR);
                    Yii::trace("Mail sent FAIL. Subject:{$notification->subject}. To:{$recipient->to}, From: {$recipient->from}", 'MailNotificator');
                }
            }
        }
    }
}
