<?php

/**
 * 
 **/
class SearchAdaptor
{
    /**
     * Number of page
     *
     * @public int
     **/
    public $pageNumber;

    /**
     * Items per page
     *
     * @public int
    **/
    public $pageSize;

    public $searchApi;

    private $_filteredResults = array();

    private $_searchTotalResults;

    private $_searchOffset = 0;

    private $_term;

    private $_params;

    private $_usePersonalization;


    public function __construct($term, $params = array(), $usePersonalization = false)
    {
        $this->_term = $term;
        $this->_params = $params;
        $this->_usePersonalization = $usePersonalization;
        $this->searchApi = new SearchApi();
    }

    /**
     * Get current page results
     *
     * @return array
     **/
    public function fetchNext() 
    {
        while((count($this->_filteredResults) < $this->pageSize * $this->pageNumber) && $this->existsMore()){
            $portion = $this->fetchPortion();
            if($portion === false){
                return array();
            } else {
                $this->_filteredResults =array_merge($this->_filteredResults, $portion) ;
            }
        }
        $next = array_slice($this->_filteredResults, $this->pageSize * $this->pageNumber - $this->pageSize, $this->pageSize);

        return $next;
    }

    public function existsMore()
    {
        if($this->_searchTotalResults === null){
            return true;
        }
        return $this->_searchTotalResults - $this->_searchOffset > 0 ||
            $this->pageSize * $this->pageNumber < count($this->_filteredResults);
    }

    protected function fetchPortion(){
        $response = $this->searchApi->search($this->_term,$this->_params, $this->_searchOffset, $this->pageSize);
        $this->_searchOffset = $response['meta']['start'] + count($response['results']);
        $this->_searchTotalResults = $response['meta']['totalhits'];
        if(!$response){
            return false;
        }
        $results = array_values($response['results']);
        if(empty($results)) {
            return false;    
        }

        if($this->_usePersonalization) {
            $results = $this->_personalize($results);
        }
        return $results;
    }

    private function _personalize($data)
    {
        // get names of personalisation fields
        $filters = Yii::app()->getModule('contentSource')->contentSource->personalisationFields;
        foreach ($data as $key=>$resultItem) {
            $persAttributes = array();
            foreach ($filters as $filter) {
                if (isset($resultItem['attributes'][$filter]) && trim($resultItem['attributes'][$filter])!=''  && trim($resultItem['attributes'][$filter])!='NULL') {
                    $persAttributes = array_merge($persAttributes, explode(Yii::app()->getModule('search')->personalisationDelimiter, $resultItem['attributes'][$filter]));
                }
            }
            /*  check if current user can see the item */
            if ( !Yii::app()->getModule('personalisation')->rule->checkAccess(Yii::app()->user->id, $persAttributes) ) {
                unset($data[$key]);
                Yii::trace('Skipped ' . $resultItem['contentid'].' Permissions: '.implode(',', $persAttributes), 'search.driver');
            }
        }
        return $data;
    }
    
}
